﻿#region Copyright © 2013 Miracle Business Solutions
// All rights are reserved. Reproduction or transmission in whole or in part,
// any form or by any means, electronic, mechanical or otherwise, is prohibited
// without the prior written consent of the copyright owner.
// 
// Document:	AutoFocus\GUI.cs
// Creation Date:	24/11/2013
// Last Edit:		24/11/2013
// Author:		Moreno Sint Hill
#endregion

using System;
using System.Drawing;
using System.Windows.Forms;
using Styx.Common;
using Styx.Helpers;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace AutoFocus
{
    using System.Linq;

    using Styx;

    public partial class GUI : Form
    {
        public GUI()
        {
            AutoFocus.AFSettings.Instance.Load();
            InitializeComponent();
            this.PopulateComboBox();
            SetComboBoxEnum(TargetBox,(int) AutoFocus.AFSettings.Instance.SelectedType);
            if (AutoFocus.AFSettings.Instance.SelectedType == AutoFocus.TargetType.TargetByGuid)
            {
                btn_Scan.Enabled = true;
                tbGUID.Enabled = true;
                tbName.Enabled = true;
            }
            else
            {
                btn_Scan.Enabled = false;
                tbGUID.Enabled = false;
                tbName.Enabled = false;
            }
            
        }

        private void OnScanClick(object sender, EventArgs e)
        {
            //Set Name
           AutoFocus.AFSettings.Instance.FocusName = tbName.Text;
            WoWUnit unit = null;
            WoWPartyMember woWPartyMember;
            switch (AutoFocus.AFSettings.Instance.SelectedType)
           {
               case AutoFocus.TargetType.Tank: 
                   woWPartyMember = StyxWoW.Me.GroupInfo.PartyMembers.FirstOrDefault(
                       a => a.Role == WoWPartyMember.GroupRole.Tank && a.IsOnline);
                   if (
                       woWPartyMember != null)
                   {
                       unit = woWPartyMember.ToPlayer();
                   }
                   break;
               case AutoFocus.TargetType.Healer:
                   woWPartyMember = StyxWoW.Me.GroupInfo.PartyMembers.FirstOrDefault(
                       a => a.Role == WoWPartyMember.GroupRole.Healer && a.IsOnline);
                   if (
                       woWPartyMember != null)
                   {
                       unit = woWPartyMember.ToPlayer();
                   }
                   break;
               case AutoFocus.TargetType.Party0:
                   unit = ObjectManager.GetObjectByGuid<WoWUnit>(StyxWoW.Me.GetPartyMemberGuid(0));
                   break;
               case AutoFocus.TargetType.Party1:
                   unit = ObjectManager.GetObjectByGuid<WoWUnit>(StyxWoW.Me.GetPartyMemberGuid(1));
                   break;
               case AutoFocus.TargetType.Party2:
                   unit = ObjectManager.GetObjectByGuid<WoWUnit>(StyxWoW.Me.GetPartyMemberGuid(2));
                   break;
               case AutoFocus.TargetType.Party3:
                   unit = ObjectManager.GetObjectByGuid<WoWUnit>(StyxWoW.Me.GetPartyMemberGuid(3));
                   break;
               case AutoFocus.TargetType.TargetByGuid:
                   unit = ObjectManager.GetObjectsOfType<WoWPlayer>()
                   .Find(a => a != null && a.IsValid && a.Name.Equals(AutoFocus.AFSettings.Instance.FocusName));
                   break;
               default:
                   throw new ArgumentOutOfRangeException();
           }

            //Search
           

            if (unit != null && unit.IsValid)
            {
                AutoFocus.AFSettings.Instance.FocusGUID = unit.Guid;
                tbGUID.Text = "Succeeded";
                tbName.Text = unit.Name;
                tbGUID.ForeColor = Color.Green;
            }
            else
            {
                tbGUID.Text = "Failed";
                tbGUID.ForeColor = Color.Red;
            }
        }

        public void PopulateComboBox()
        {
            TargetBox.Items.Add(new CboItem((int)AutoFocus.TargetType.Tank, "Tank"));
            TargetBox.Items.Add(new CboItem((int)AutoFocus.TargetType.Healer, "Healer"));
            TargetBox.Items.Add(new CboItem((int)AutoFocus.TargetType.TargetByGuid, "TargetByGuid"));
            TargetBox.Items.Add(new CboItem((int)AutoFocus.TargetType.Party3, "Party Member 4"));
            TargetBox.Items.Add(new CboItem((int)AutoFocus.TargetType.Party0, "Party Member 1"));
            TargetBox.Items.Add(new CboItem((int)AutoFocus.TargetType.Party1, "Party Member 2"));
            TargetBox.Items.Add(new CboItem((int)AutoFocus.TargetType.Party2, "Party Member 3"));
        }   

        private static void SetComboBoxEnum(ComboBox cb, int e)
        {
            try
            {
                CboItem item;
                for (int i = 0; i < cb.Items.Count; i++)
                {
                    item = (CboItem)cb.Items[i];
                    if (item.E != e) continue;
                    cb.SelectedIndex = i;
                    return;
                }
                item = (CboItem)cb.Items[0];
                if (item != null)
                {
                    Logging.Write("Dialog Error: combobox {0} does not have enum({1}) in list, defaulting to enum({2})",
                   cb.Name, e, item.E);
                }
                else Logging.Write("Dialog Error: combobox {0} does not have enum({1}) in list, defaulting to enum(null)",
                 cb.Name, e);
                cb.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
             Logging.WriteException(ex.InnerException);
            }

        }
        
        private static int GetComboBoxEnum(ComboBox cb)
        {
            try
            {
                var item = (CboItem)cb.Items[cb.SelectedIndex];
                return item.E;
            }
            catch (Exception ex)
            {
                Logging.WriteException(ex.InnerException);
            }

            return 0;
        }

        internal class CboItem
        {
            public readonly int E;
            private readonly string _s;

            public CboItem(int pe, string ps)
            {
                E = pe;
                _s = ps;
            }

            public override string ToString()
            {
                return _s;
            }
        }

        private void TargetTypeChanged(object sender, EventArgs e)
        {
            AutoFocus.AFSettings.Instance.SelectedType = (AutoFocus.TargetType) GetComboBoxEnum(TargetBox);
            switch (AutoFocus.AFSettings.Instance.SelectedType)
            {
                case AutoFocus.TargetType.Tank:
                btn_Scan.Enabled = true;
                    break;
                case AutoFocus.TargetType.Healer:
                btn_Scan.Enabled = true;
                    break;
                case AutoFocus.TargetType.Party0:
                btn_Scan.Enabled = true;
                    break;
                case AutoFocus.TargetType.Party1:
                btn_Scan.Enabled = true;
                    break;
                case AutoFocus.TargetType.Party2:
                btn_Scan.Enabled = true;
                    break;
                case AutoFocus.TargetType.Party3:
                btn_Scan.Enabled = true;
                    break;
                case AutoFocus.TargetType.TargetByGuid:   
                btn_Scan.Enabled = true;
                tbGUID.Enabled = true;
                tbName.Enabled = true;
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }
    }
}
